#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "heider"
__doc__ = r"""

           Created on 5/5/22
           """

from pathlib import Path
from typing import List

from jord.gdal_utilities.enums import GdalAccessEnum
from jord.gdal_utilities.importing import GDAL, OSR

__all__ = [
    "get_georeference_from_file",
    "set_georeference_to_file",
    "geotiff_to_tiff",
    "tiff_to_geotiff",
    "copy_gcps_to_file",
]


def get_georeference_from_file(file: Path) -> List[GDAL.GCP]:
    """

    :param file:
    :type file:
    :return:
    :rtype:
    """
    with GDAL.Open(file) as f:
        # return ds.GetProjection()
        # return f.GetGCPs()
        return f.GetGeoTransform()


def set_georeference_to_file(file: Path, gcps: List[GDAL.GCP]) -> None:
    """

    :param file:
    :type file:
    :param gcps:
    :type gcps:

    TODO: FINISH!

    """
    img_path = "/path/to/tif"

    # input values for geotransform
    ulx = 25
    uly = 60
    xres = 0.5
    yres = -0.5
    xrot = 0
    yrot = 0

    geotransform = (ulx, xres, xrot, uly, yrot, yres)

    # find projection
    srs = OSR.SpatialReference()
    srs.ImportFromEPSG(4326)

    # update image georeference
    ds = GDAL.Open(img_path, GdalAccessEnum.update.value)
    ds.SetGeoTransform(geotransform)
    ds.SetProjection(srs.ExportToWkt())
    ds.FlushCache()
    del ds

    # do this for each image and then stack


def copy_gcps_to_file(file: Path, gcps: List[GDAL.GCP]) -> None:
    """

    :param file:
    :type file:
    :param gcps:
    :type gcps:
    :rtype:None
    """

    with GDAL.Open(file, GdalAccessEnum.update.value) as f:
        f.SetGCPs(gcps)
        f.FlushCache()


def geotiff_to_tiff(src: Path, dst: Path) -> None:
    """
    https://gdal.org/user/translate.html

    :rtype: None
    """
    GDAL.Translate(
        dst,
        src,
        format="JPEG",
        outputType=GDAL.GDT_Byte,
        creationOptions=["COMPRESS=LZW"],
    )


def tiff_to_geotiff(src: Path, dst: Path) -> None:
    """
    https://gdal.org/user/translate.html

    :rtype: None
    """
    GDAL.Translate(
        dst,
        src,
        format="GTiff",
        outputType=GDAL.GDT_Byte,
        creationOptions=["COMPRESS=LZW"],
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from typing import Union, Tuple, List, Iterable

import numpy
from numpy import array
from scipy.spatial import Voronoi
from shapely.geometry import LineString, MultiLineString, MultiPolygon, Polygon
from shapely.geometry.base import BaseGeometry
from shapely.ops import unary_union
from warg import Number

__all__ = ["find_centerline"]

from jord.shapely_utilities.polygons import polygon_has_interior_rings, iter_polygons


def find_centerline(
    input_geometry: Union[Polygon, MultiPolygon], step_size: Number = 0.5
) -> MultiLineString:
    """


    :param input_geometry: input geometry
    :type input_geometry: Union[Polygon, MultiPolygon]
    :param step_size: densify the input geometry border by placing additional points at this distance, in meters
    :type step_size: Optional[float]
    :raises NotImplementedError: input geometry is not of type Union[Polygon, MultiPolygon]
    """

    if not (
        isinstance(input_geometry, Polygon) or isinstance(input_geometry, MultiPolygon)
    ):
        raise NotImplementedError

    step_size = abs(step_size)

    ext_xy = input_geometry.envelope.exterior.xy
    _min_x, _min_y = (
        int(min(ext_xy[0])),
        int(min(ext_xy[1])),
    )

    vertices, ridges = _get_voronoi_vertices_and_ridges(
        input_geometry, step_size, minx=_min_x, miny=_min_y
    )

    lines = []
    for ridge in ridges:
        if _ridge_is_finite(ridge):
            starting_point = _create_point_with_restored_coordinates(
                x=vertices[ridge[0]][0],
                y=vertices[ridge[0]][1],
                _min_x=_min_x,
                _min_y=_min_y,
            )
            ending_point = _create_point_with_restored_coordinates(
                x=vertices[ridge[1]][0],
                y=vertices[ridge[1]][1],
                _min_x=_min_x,
                _min_y=_min_y,
            )
            linestring = LineString((starting_point, ending_point))

            if _linestring_is_within_input_geometry(linestring, input_geometry):
                lines.append(linestring)

    if len(lines) < 2:
        raise Exception("too few ridges")

    return MultiLineString(lines=unary_union(lines))


def _get_voronoi_vertices_and_ridges(
    _input_geometry: BaseGeometry,
    _step_size: float,
    minx: float,
    miny: float,
) -> Tuple[numpy.ndarray, List[List[int]]]:
    borders = _get_densified_borders(_input_geometry, _step_size, minx, miny)

    voronoi_diagram = Voronoi(borders)
    return voronoi_diagram.vertices, voronoi_diagram.ridge_vertices


def _ridge_is_finite(ridge: Iterable) -> bool:
    return -1 not in ridge


def _create_point_with_restored_coordinates(
    x: float, y: float, _min_x: float, _min_y: float
) -> Tuple[float, float]:
    return (x + _min_x, y + _min_y)


def _linestring_is_within_input_geometry(
    linestring: LineString, input_geometry: BaseGeometry
) -> bool:
    return linestring.within(input_geometry) and len(linestring.coords[0]) > 1


def _get_densified_borders(
    _input_geometry: BaseGeometry, _step_size, minx: float, miny: float
) -> numpy.ndarray:
    polygons = iter_polygons(_input_geometry)
    points = []
    for polygon in polygons:
        points += _get_interpolated_boundary(polygon.exterior, _step_size, minx, miny)
        if polygon_has_interior_rings(polygon):
            for interior in polygon.interiors:
                points += _get_interpolated_boundary(
                    interior, _step_size, minx=minx, miny=miny
                )

    return array(points)


def _get_interpolated_boundary(
    boundary: BaseGeometry, _step_size: float, minx: float, miny: float
) -> List[Tuple[float, float]]:
    line = LineString(boundary)

    return (
        [_get_coordinates_of_first_point(line, minx, miny)]
        + _get_coordinates_of_interpolated_points(
            line, _step_size, min_x=minx, min_y=miny
        )
        + [_get_coordinates_of_last_point(line, minx=minx, miny=miny)]
    )


def _create_point_with_reduced_coordinates(
    x: float, y: float, _min_x: float, _min_y: float
) -> Tuple[float, float]:
    return (x - _min_x, y - _min_y)


def _get_coordinates_of_first_point(
    linestring: LineString, minx: float, miny: float
) -> Tuple[float, float]:
    return _create_point_with_reduced_coordinates(
        x=linestring.xy[0][0], y=linestring.xy[1][0], _min_x=minx, _min_y=miny
    )


def _get_coordinates_of_last_point(
    linestring: LineString, minx: float, miny: float
) -> Tuple[float, float]:
    return _create_point_with_reduced_coordinates(
        x=linestring.xy[0][-1], y=linestring.xy[1][-1], _min_x=minx, _min_y=miny
    )


def _get_coordinates_of_interpolated_points(
    linestring: LineString, _step_size: Number, min_x: float, min_y: float
) -> List[Tuple[float, float]]:
    interpolation_distance = _step_size
    intermediate_points = []
    while interpolation_distance < linestring.length:
        point = linestring.interpolate(interpolation_distance)
        reduced_point = _create_point_with_reduced_coordinates(
            x=point.x, y=point.y, _min_x=min_x, _min_y=min_y
        )
        intermediate_points.append(reduced_point)
        interpolation_distance += _step_size

    return intermediate_points


if __name__ == "__main__":

    def uashdua():
        from shapely.geometry import Polygon
        from jord.qlive_utilities import AutoQliveClient

        polygon = Polygon([[0, 0], [0, 4], [4, 4], [4, 0]])

        with AutoQliveClient() as qlive:
            qlive.add_shapely_geometry(polygon)
            qlive.add_shapely_geometry(find_centerline(polygon))

    uashdua()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Lindbjerg"
__doc__ = r"""

           Created on 02-12-2020
           """

import cgi
import subprocess
from pathlib import Path

__all__ = ["install_requirements_from_file", "install_requirements_from_name"]

from typing import Iterable, Tuple, Optional
import os
from subprocess import check_output
import sys

SP_CALLABLE = subprocess.check_call  # subprocess.call


def install_requirements_from_file(requirements_path: Path) -> None:
    """
    Install requirements from a requirements.txt file.

    :param requirements_path: Path to requirements.txt file.

    """

    # pip.main(["install", "pip", "--upgrade"]) # REQUIRES RESTART OF QGIS

    args = ["install", "-r", str(requirements_path), "--upgrade"]
    # args = ["install", "rasterio", "--upgrade"] # RASTERIO for window DOES NOT WORK ATM, should be installed manually

    if False:
        import pip

        pip.main(args)

    elif False:
        SP_CALLABLE(["pip"] + args)

    elif True:
        SP_CALLABLE(["python", "-m", "pip"] + args)


def is_requirement_installed(requirement_name: str) -> bool:
    if requirement_has_version(requirement_name):
        if get_requirement_version(requirement_name) == get_installed_version(
            requirement_name
        ):
            return True
    if get_installed_version(requirement_name):
        return True
    return False


def requirement_has_version(requirement_name: str) -> bool:
    return get_requirement_version(requirement_name) is not None


def get_requirement_version(requirement_name: str) -> Optional[str]:
    s = requirement_name.split("==")
    if len(s) == 2:
        return s[-1]
    return None


def get_installed_version(requirement_name: str) -> str:
    import pkg_resources

    try:
        dist = pkg_resources.get_distribution(requirement_name)
        if dist:
            return dist.parsed_version  # .version
    except pkg_resources.DistributionNotFound as e:
        pass
    return None


def get_newest_version(
    requirement_name: str,
    pip_index=os.environ.get("PIP_INDEX_URL", "https://pypi.org/pypi/"),
) -> str:
    """

    :param requirement_name:
    :param pip_index:
    :return:
    """
    from pkg_resources import parse_version

    import json
    from urllib.request import Request, urlopen

    def get_charset(headers, default: str = "utf-8"):
        # this is annoying.
        try:
            charset = headers.get_content_charset(default)
        except AttributeError:
            # Python 2
            charset = headers.getparam("charset")
            if charset is None:
                ct_header = headers.getheader("Content-Type")
                content_type, params = cgi.parse_header(ct_header)
                charset = params.get("charset", default)
        return charset

    def json_get(url: str, headers: Tuple = (("Accept", "application/json"),)):
        request = Request(url=url, headers=dict(headers))
        response = urlopen(request)
        code = response.code
        if code != 200:
            err = ConnectionError(f"Unexpected response code {code}")
            err.response_data = response.read()
            raise err
        raw_data = response.read()
        response_encoding = get_charset(response.headers)
        decoded_data = raw_data.decode(response_encoding)
        data = json.loads(decoded_data)
        return data

    def get_data_pypi(name: str, index: str = pip_index):
        uri = f"{index.rstrip('/')}/{name.split('[')[0]}/json"
        data = json_get(uri)
        return data

    def get_versions_pypi(name: str, index: str = pip_index):
        data = get_data_pypi(name, index)
        version_numbers = sorted(data["releases"], key=parse_version)
        return tuple(version_numbers)

    return parse_version(get_versions_pypi(requirement_name)[-1])


def is_requirement_updatable(requirement_name: str) -> bool:
    if not is_requirement_installed(requirement_name):
        return True

    if requirement_has_version(requirement_name):
        if get_requirement_version(requirement_name) != get_installed_version(
            requirement_name
        ):
            return True

    if get_newest_version(requirement_name) > get_installed_version(requirement_name):
        return True

    return False


def install_requirements_from_name(*requirements_name: Iterable[str]) -> None:
    """
    Install requirements from names.

    :param requirements_name: Name of requirements.
    """
    # pip.main(["install", "pip", "--upgrade"]) # REQUIRES RESTART OF QGIS

    # if isinstance(requirements_name, Iterable) and len(requirements_name)==1:
    # ... # handle wrong input format

    args = ["install", "-U", *requirements_name]
    # args = ["install", "rasterio", "--upgrade"] # RASTERIO for window DOES NOT WORK ATM, should be installed manually

    if False:
        import pip

        pip.main(args)

    elif False:
        SP_CALLABLE(["pip"] + args)
    # subprocess.check_call([sys.executable, '-m', 'conda', 'install', '<packagename>'])
    elif True:
        interpreter = Path(sys.executable).absolute()
        SP_CALLABLE([str(interpreter), "-m", "pip"] + args)


def remove_requirements_from_name(
    *requirements_name: Iterable[str], num_repeat: int = 1
) -> None:
    """
    Multiple colliding versions may be installed at once, (conda, pip, ....)

    :param num_repeat:     Repeat arg let you choose how many times to try to uninstall the packages.
    :param requirements_name:
    :return:
    """

    args = ["uninstall", "-y", *requirements_name]

    for _ in range(num_repeat):
        if False:
            import pip

            pip.main(args)

        elif False:
            SP_CALLABLE(["pip"] + args)

        elif True:
            interpreter = Path(sys.executable).absolute()
            SP_CALLABLE(
                [str(interpreter), "-m", "pip"] + args
            )  # figure out which python!

            for r in requirements_name:  # assuming no name aliasing
                del requirements_name  # unload now deleted modules

            reqs = check_output([sys.executable, "-m", "pip", "freeze"])
            # installed_packages = [r.decode().split('==')[0] for r in reqs.split()]
            # print(installed_packages)


if __name__ == "__main__":
    print(
        get_newest_version("warg"),
        get_installed_version("warg"),
        get_newest_version("warg") == get_installed_version("warg"),
    )

    install_requirements_from_name("that")
    remove_requirements_from_name("that")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Lindbjerg"
__doc__ = r"""

           Created on 02-12-2020
           """

from typing import Sequence

import numpy
from PIL import Image

# noinspection PyUnresolvedReferences
from qgis.PyQt import QtGui

# noinspection PyUnresolvedReferences
from qgis.core import (
    QgsCoordinateReferenceSystem,
    QgsCoordinateTransform,
    QgsPoint,
    QgsVectorLayer,
)

__all__ = [
    "get_qimage_from_numpy",
    "transform_coordinates",
    "get_coordinates_of_layer_extent",
]


def get_qimage_from_numpy(img: Image, debug: bool = False) -> QtGui.QImage:
    """

    :param img:
    :param debug:
    :return:
    """
    # if isinstance(img, Image):
    #    img = img.data
    # if isinstance(img, numpy.ndarray):
    #    img = img.data

    # if isinstance(img, cv2.Image):
    #    img = img.data # QtGui.QImage.Format_BGR888
    # aformat = QImage.Format_RGB32

    img = img.data
    height, width, channels = img.shape

    if debug:
        print(f"height: {height}, width: {width}, channels: {channels}")

    bytes_per_line = channels * width
    img = numpy.require(img, numpy.uint8, "C")

    return QtGui.QImage(img, width, height, bytes_per_line, QtGui.QImage.Format_RGB888)


def transform_coordinates(coordinates: Sequence, from_crs: str, to_crs: str) -> list:
    """

    function to transform a set of coordinates from one CRS to another


    :param coordinates:
    :param from_crs:
    :param to_crs:
    :return:"""
    crs_src = QgsCoordinateReferenceSystem(from_crs)
    crs_dest = QgsCoordinateReferenceSystem(to_crs)
    xform = QgsCoordinateTransform(crs_src, crs_dest)

    coordinates_as_points = [
        QgsPoint(coordinates[0], coordinates[1]),
        QgsPoint(coordinates[2], coordinates[3]),
    ]  # convert list of coordinates to QgsPoint objects

    transformed_coordinates_as_points = [
        xform.transform(point) for point in coordinates_as_points
    ]  # do transformation for each point

    return [
        transformed_coordinates_as_points[0].x(),
        transformed_coordinates_as_points[0].y(),
        transformed_coordinates_as_points[1].x(),
        transformed_coordinates_as_points[1].y(),
    ]  # transform the QgsPoint objects back to a list of coordinates


def get_coordinates_of_layer_extent(layer: QgsVectorLayer) -> list:
    """

    function to get coordinates of a layer extent


    :param layer:
    :return:

    """

    layer_rectangle = layer.extent()

    return [
        layer_rectangle.xMinimum(),
        layer_rectangle.yMinimum(),
        layer_rectangle.xMaximum(),
        layer_rectangle.yMaximum(),
    ]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from typing import Any

from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QAction, QWidget

from jord.qgis_utilities.helpers.signals import reconnect_signal

__all__ = ["create_action"]


def create_action(
    icon_path: str,
    text: str,
    callback: callable,
    enabled_flag: bool = True,
    toolbar: Any = None,
    iface: Any = None,
    menu: Any = None,
    status_tip: str = None,
    whats_this: str = None,
    parent: QWidget = None,
) -> QAction:
    """Action creation factory

    Add a toolbar icon to the toolbar.

      :param icon_path: Path to the icon for this action. Maybe a resource
          path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
      :type icon_path: Str

      :param text: Text that should be shown in menu items for this action.
      :type text: Str

      :param callback: Function to be called when the action is triggered.
      :type callback: Callable

      :param enabled_flag: A flag indicating if the action should be enabled
          by default. Defaults to True.
      :type enabled_flag: Bool

      :param toolbar: Flag indicating whether the action should also
          be added to the menu. Defaults to True.
      :type toolbar: Any

      :param iface: Flag indicating whether the action should also
          be added to the toolbar. Defaults to True.
      :type iface: Any

      :param menu: Flag indicating whether the action should also
          be added to the toolbar. Defaults to True.
      :type menu: Any

      :param status_tip: Optional text to show in a popup when the mouse pointer hovers over the action.
      :type status_tip: Str

      :param parent: Parent widget for the new action. Defaults None.
      :type parent: QWidget

      :param whats_this: Optional text to show in the status bar when the
          mouse pointer hovers over the action.

      :returns: The action that was created. Note that the action is also
          added to .actions list.
      :rtype: QAction
    """

    icon = QIcon(icon_path)
    action = QAction(icon, text, parent)
    reconnect_signal(action.triggered, callback)
    action.setEnabled(enabled_flag)

    if status_tip is not None:
        action.setStatusTip(status_tip)

    if whats_this is not None:
        action.setWhatsThis(whats_this)

    if toolbar:
        toolbar.addAction(action)

    if iface:
        assert menu
        iface.addPluginToMenu(menu, action)

    return action



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Lindbjerg"
__doc__ = r"""

           Created on 02-12-2020
           """

# noinspection PyUnresolvedReferences
from qgis.PyQt import QtGui

# noinspection PyUnresolvedReferences
from qgis.PyQt.QtCore import Qt

# noinspection PyUnresolvedReferences
from qgis.core import QgsWkbTypes

# noinspection PyUnresolvedReferences
from qgis.gui import QgsMapCanvas, QgsRubberBand

__all__ = ["make_rubber_band"]


def make_rubber_band(canvas: QgsMapCanvas) -> QgsRubberBand:
    """
    Create a rubber band on the canvas.

    :param canvas: The canvas to create the rubber band on.
    :return: The rubber band.
    """
    fill_color = QtGui.QColor("white")
    fill_color.setAlphaF(0.0)

    stroke_color = QtGui.QColor("red")
    stroke_color.setAlphaF(0.65)

    rubber_band = QgsRubberBand(canvas, QgsWkbTypes.PolygonGeometry)
    rubber_band.setWidth(2)
    rubber_band.setColor(fill_color)  # fill color
    rubber_band.setStrokeColor(stroke_color)  # stroke color
    rubber_band.setLineStyle(Qt.DotLine)

    return rubber_band



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# noinspection PyUnresolvedReferences
from pyplugin_installer.installer_data import repositories, plugins

__all__ = ["plugin_status"]


def plugin_status(plugin_key: str) -> str:
    all_plugins = plugins.all()
    if plugin_key in all_plugins:
        return all_plugins[plugin_key]["status"]
    return "Not Found"


# utils.reloadPlugin()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from typing import Union

from .importing import GDAL, OGR

__all__ = ["clone_data_to_memory", "clone_raster_to_memory", "clone_vector_to_memory"]


def clone_data_to_memory(
    ds: Union[GDAL.Dataset, OGR.DataSource], name: str = ""
) -> Union[GDAL.Dataset, OGR.DataSource]:
    """

    :param ds:
    :param name:
    :return:
    """
    if isinstance(ds, GDAL.Dataset):
        return clone_raster_to_memory(ds, name)
    elif isinstance(ds, OGR.DataSource):
        return clone_vector_to_memory(ds, name)
    else:
        raise TypeError("Data source must be of GDAL dataset or OGR datasource")


def clone_vector_to_memory(vector_ds: OGR.DataSource, name: str = "") -> OGR.DataSource:
    """

    :param vector_ds:
    :param name:
    :return:
    """
    return OGR.GetDriverByName("Memory").CopyDataSource(vector_ds, name)


def clone_raster_to_memory(raster_ds: GDAL.Dataset, name: str = "") -> GDAL.Dataset:
    """

    :param raster_ds:
    :param name:
    :return:
    """
    return GDAL.GetDriverByName("MEM").CopyDataSource(raster_ds, name)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pathlib import Path

import pandas
from shapely import wkb

__all__ = ["load_wkbs_from_csv", "csv_wkt_generator"]


def load_wkbs_from_csv(csv_file_path: Path, geometry_column: str = "Shape") -> wkb:
    """
    Well-Known Text
    """
    return pandas.read_csv(str(csv_file_path))[geometry_column].apply(wkb.loads)


def csv_wkt_generator(csv_file_path: Path, geometry_column: str = "Shape") -> wkb:
    """

    :param csv_file_path:
    :param geometry_column:
    :return:
    """
    for idx, g in pandas.read_csv(
        str(csv_file_path), usecols=[geometry_column]
    ).iterrows():
        yield wkb.loads(g)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from typing import Mapping

from PyQt5.Qt import QColor

# noinspection PyUnresolvedReferences
from qgis.core import (
    QgsVectorLayer,
    QgsSymbol,
    QgsRendererCategory,
    QgsCategorizedSymbolRenderer,
    QgsLineSymbol,
)
from warg import TripleNumber

__all__ = ["style_layer_from_mapping"]


def style_layer_from_mapping(
    layer: QgsVectorLayer,
    style_mapping_field_dict: Mapping,
    field_name: str = "layer",
    default_color: TripleNumber = (0, 0, 0),
    default_opacity: float = 1.0,
    default_width: float = 0.1,
) -> None:
    style_mapping = style_mapping_field_dict[field_name]

    render_categories = []
    for cat in layer.uniqueValues(layer.fields().indexFromName(field_name)):
        cat_color = default_color
        cat_opacity = default_opacity
        cat_width = default_width
        label = str(cat)

        if cat in style_mapping.keys():
            style = style_mapping[label]
            if "color" in style:
                cat_color = (
                    int(n) for n in style["color"]
                )  # TODO: also support with AlphaChannel | Qt.GlobalColor | QGradient
            if "opacity" in style:
                cat_opacity = max(0.0, min(float(style["opacity"]), 1.0))
            if "width" in style:
                cat_width = max(0.0, float(style["width"]))

        symbol = QgsSymbol.defaultSymbol(layer.geometryType())
        symbol.setColor(QColor(*(cat_color), 255))
        symbol.setOpacity(cat_opacity)

        if isinstance(symbol, QgsLineSymbol):
            symbol.setWidth(cat_width)
        else:
            print(f"width ignored, symbol is of type: {type(symbol)}")

        render_categories.append(
            QgsRendererCategory(cat, symbol=symbol, label=label, render=True)
        )

    layer.setRenderer(QgsCategorizedSymbolRenderer(field_name, render_categories))
    layer.triggerRepaint()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from enum import Enum

# noinspection PyUnresolvedReferences
from qgis.core import (
    QgsMultiBandColorRenderer,
    QgsPalettedRasterRenderer,
    QgsSingleBandColorDataRenderer,
    QgsSingleBandGrayRenderer,
    QgsSingleBandPseudoColorRenderer,
)

__all__ = ["QgisRendererEnum"]

from jord.geojson_utilities import GeoJsonGeometryTypesEnum


class QgisRendererEnum(Enum):
    multi_band = QgsMultiBandColorRenderer
    paletted_raster = QgsPalettedRasterRenderer
    single_band_color = QgsSingleBandColorDataRenderer
    single_band_gray = QgsSingleBandGrayRenderer
    single_band_pseudo = QgsSingleBandPseudoColorRenderer


class QgisLayerTypeEnum(Enum):
    point = GeoJsonGeometryTypesEnum.point.value.__name__
    multi_point = GeoJsonGeometryTypesEnum.multi_point.value.__name__
    line_string = GeoJsonGeometryTypesEnum.line_string.value.__name__
    multi_line_string = GeoJsonGeometryTypesEnum.multi_line_string.value.__name__
    polygon = GeoJsonGeometryTypesEnum.polygon.value.__name__
    multi_polygon = GeoJsonGeometryTypesEnum.multi_polygon.value.__name__
    curve_polygon = "CurvePolygon"
    multi_surface = "MultiSurface"
    compound_curve = "CompoundCurve"
    multi_curve = "MultiCurve"
    no_geometry = "No Geometry"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from enum import Enum
from typing import Any

import numpy
from qgis.core import Qgis


class QgisDataTypeEnum(Enum):
    unknown = Qgis.UnknownDataType  # Unknown or unspecified type.

    # int8 = Qgis.Int8  # Eight bit signed integer (qint8) (added in QGIS 3.30)
    int16 = Qgis.Int16  # Sixteen bit signed integer (qint16)
    int32 = Qgis.Int32  # Thirty two bit signed integer (qint32)

    uint8 = Qgis.Byte  # Eight bit unsigned integer (quint8)
    uint16 = Qgis.UInt16  # Sixteen bit unsigned integer (quint16)
    uint32 = Qgis.UInt32  # Thirty two bit unsigned integer (quint32)

    float32 = Qgis.Float32  # Thirty two bit floating point (float)
    float64 = Qgis.Float64  # Sixty four bit floating point (double)

    cint16 = Qgis.CInt16  # Complex Int16.
    cint32 = Qgis.CInt32  # Complex Int32.

    cfloat32 = Qgis.CFloat32  # Complex Float32.
    cfloat64 = Qgis.CFloat64  # Complex Float64.

    argb32 = (
        Qgis.ARGB32
    )  # Color, alpha, red, green, blue, 4 bytes the same as QImage::Format_ARGB32.
    argb32pre = (
        Qgis.ARGB32_Premultiplied
    )  # Color, alpha, red, green, blue, 4 bytes the same as QImage::Format_ARGB32_Premultiplied.


NUMPY_TO_QGIS_TYPE_MAPPING = {  # Numpy type         C type      Description
    numpy.bool_: QgisDataTypeEnum.uint8,  # bool Boolean (True or False) stored as a byte
    numpy.int_: QgisDataTypeEnum.int32,  # long Platform-defined
    numpy.complex_: QgisDataTypeEnum.unknown,
    numpy.float_: QgisDataTypeEnum.unknown,
    # numpy.byte: QgisDataTypeEnum.int8,  # signed char Platform-defined
    numpy.ubyte: QgisDataTypeEnum.uint8,  # unsigned char Platform-defined
    # numpy.short: QgisDataTypeEnum.int8,  # short Platform-defined
    numpy.intc: QgisDataTypeEnum.cint16,  # int Platform-defined
    # numpy.int8: QgisDataTypeEnum.int8,
    numpy.intp: QgisDataTypeEnum.unknown,
    numpy.int16: QgisDataTypeEnum.int16,
    numpy.int32: QgisDataTypeEnum.int32,
    numpy.int64: QgisDataTypeEnum.unknown,
    numpy.longlong: QgisDataTypeEnum.unknown,  # long long Platform-defined
    numpy.ushort: QgisDataTypeEnum.uint8,  # unsigned short Platform-defined
    numpy.uintc: QgisDataTypeEnum.unknown,  # unsigned int Platform-defined
    numpy.uint: QgisDataTypeEnum.uint32,  # unsigned long Platform-defined
    numpy.uint8: QgisDataTypeEnum.uint8,
    numpy.uint16: QgisDataTypeEnum.uint16,
    numpy.uint32: QgisDataTypeEnum.uint32,
    numpy.uint64: QgisDataTypeEnum.unknown,
    numpy.ulonglong: QgisDataTypeEnum.unknown,  # unsigned long long Platform-defined
    numpy.half: QgisDataTypeEnum.unknown,  # Half precision float: sign bit, 5 bits exponent, 10 bits mantissa
    numpy.float16: QgisDataTypeEnum.unknown,  # Half precision float: sign bit, 5 bits exponent, 10 bits mantissa
    numpy.float32: QgisDataTypeEnum.float32,
    numpy.float64: QgisDataTypeEnum.float64,
    numpy.single: QgisDataTypeEnum.float32,  # float Platform-defined single precision float: typically sign bit,
    # 8 bits exponent,  # 23 bits mantissa
    numpy.double: QgisDataTypeEnum.float64,  # double Platform-defined double precision float: typically sign bit,
    # 11 bits exponent, 52 bits mantissa.
    numpy.longdouble: QgisDataTypeEnum.unknown,  # long double Platform-defined extended-precision float
    numpy.csingle: QgisDataTypeEnum.cfloat32,  # float complex Complex number, represented by two single-precision
    # floats (
    # real and imaginary components)
    numpy.cdouble: QgisDataTypeEnum.cfloat64,  # double complex Complex number, represented by two double-precision
    # floats (
    # real and imaginary
    # components).
    numpy.clongdouble: QgisDataTypeEnum.unknown,
    # long double complex  # Complex number, represented by two extended-precision floats (real and imaginary components).
    numpy.complex64: QgisDataTypeEnum.unknown,
    numpy.complex128: QgisDataTypeEnum.unknown,
}


def get_qgis_type(dtype: numpy.dtype) -> Any:
    """

    :param dtype:
    :return:
    """
    return NUMPY_TO_QGIS_TYPE_MAPPING[dtype.type]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from typing import Any

import zmq
from draugr.python_utilities import in_docker
from warg import AlsoDecorator

__all__ = ["QliveClient"]

DEFAULT_PORT = "5555"


def default_address() -> str:
    if in_docker():
        return f"tcp://host.docker.internal:{DEFAULT_PORT}"
    return f"tcp://localhost:{DEFAULT_PORT}"


class QliveClient(AlsoDecorator):
    """
    TODO: MAYBE NOT ALSO A DECORATOR

    Client for sending data to qgis instance
    """

    def __init__(self, address: str = default_address()):
        self.context = zmq.Context()
        self.socket = self.context.socket(zmq.REQ)

        if not address:
            address = default_address()

        if str.isnumeric(address):  # only port was given
            address = f"{default_address().split(f':{DEFAULT_PORT}')[0]}:{address}"

        if "tcp://" not in address:  # protocol is missing
            address = f"tcp://{address}"

        self.address = address

        self.blocking = False
        self.flag = None
        if not self.blocking:
            self.poller = zmq.Poller()
            self.flag = zmq.NOBLOCK
            self.wait_time = 1000
            self.poller.register(self.socket, zmq.POLLIN)

    def __enter__(self):
        self.socket.connect(self.address)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.socket.close()

    def send(self, *args) -> Any:
        self.socket.send(*args, self.flag)

        if self.blocking:
            return self.socket.recv()

        socks = dict(self.poller.poll(self.wait_time))
        if socks:
            if socks.get(self.socket) == zmq.POLLIN:
                return self.socket.recv(self.flag)

        return  # TODO: just ignore for now
        raise TimeoutError



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import time
import uuid
from enum import Enum
from typing import Mapping, Any, Tuple, Iterable

import numpy
import shapely.geometry.base
import tqdm
from pandas import DataFrame
from shapely.geometry.base import BaseGeometry
from warg import ensure_existence
from warg import passes_kws_to, Number

from jord import PROJECT_APP_PATH
from jord.qlive_utilities.qgis_layer_creation import (
    add_qgis_single_feature_layer,
    add_qgis_multi_feature_layer,
)

APPEND_TIMESTAMP = True
SKIP_MEMORY_LAYER_CHECK_AT_CLOSE = True
PIXEL_SIZE = 1
DEFAULT_NUMBER = 0
CONTRAST_ENHANCE = True
DEFAULT_LAYER_NAME = "QliveLayer"
DEFAULT_LAYER_CRS = "EPSG:4326"
VERBOSE = False

__all__ = [
    "add_raster",
    "add_rasters",
    "add_wkt",
    "add_wkts",
    "add_wkt_layer",
    "add_wkb",
    "add_wkbs",
    "add_wkb_layer",
    "add_dataframe",
    "add_dataframes",
    "add_dataframe_layer",
    "add_geojson",
    "add_geojsons",
    "add_geojson_layer",
    "add_shapely_geometry",
    "add_shapely_geometries",
    "add_shapely_layer",
    "clear_all",
    "remove_layers",
    "remove_layer",
    "QliveRPCMethodEnum",
    "QliveRPCMethodMap",
]


@passes_kws_to(add_qgis_single_feature_layer)
def add_wkb(qgis_instance_handle: Any, wkb: str, *args, **kwargs) -> None:
    """

    :param qgis_instance_handle:
    :param wkb:
    :param kwargs:
    :return:
    """
    # noinspection PyUnresolvedReferences
    from qgis.core import QgsGeometry

    add_qgis_single_feature_layer(
        qgis_instance_handle, QgsGeometry.fromWkb(wkb), *args, **kwargs
    )


@passes_kws_to(add_wkb)
def add_wkbs(
    qgis_instance_handle: Any, wkbs: Mapping[str, str], *args, **kwargs
) -> None:
    """

    :param qgis_instance_handle:
    :param wkbs:
    :param kwargs:
    :return:
    """
    for layer_name, wkb in wkbs.items():
        add_wkb(qgis_instance_handle, wkb, *args, name=layer_name, **kwargs)


@passes_kws_to(add_qgis_multi_feature_layer)
def add_wkb_layer(
    qgis_instance_handle: Any, wkbs: Iterable[str], *args, **kwargs
) -> None:
    # noinspection PyUnresolvedReferences
    from qgis.core import QgsGeometry

    add_qgis_multi_feature_layer(
        qgis_instance_handle,
        [QgsGeometry.fromWkb(wkb) for wkb in wkbs],
        *args,
        **kwargs,
    )


@passes_kws_to(add_qgis_single_feature_layer)
def add_wkt(qgis_instance_handle: Any, wkt: str, *args, **kwargs) -> None:
    """

    :param qgis_instance_handle:
    :param wkt:
    :param kwargs:
    :return:
    """
    # noinspection PyUnresolvedReferences
    from qgis.core import QgsGeometry

    add_qgis_single_feature_layer(
        qgis_instance_handle, QgsGeometry.fromWkt(wkt), *args, **kwargs
    )


@passes_kws_to(add_wkt)
def add_wkts(
    qgis_instance_handle: Any, wkts: Mapping[str, str], *args, **kwargs
) -> None:
    """

    :param qgis_instance_handle:
    :param wkts:
    :param kwargs:
    :return:
    """
    for layer_name, wkt in wkts.items():
        add_wkt(qgis_instance_handle, wkt, *args, name=layer_name, **kwargs)


@passes_kws_to(add_qgis_multi_feature_layer)
def add_wkt_layer(
    qgis_instance_handle: Any, wkts: Iterable[str], *args, **kwargs
) -> None:
    # noinspection PyUnresolvedReferences
    from qgis.core import QgsGeometry

    add_qgis_multi_feature_layer(
        qgis_instance_handle,
        [QgsGeometry.fromWkt(wkt) for wkt in wkts],
        *args,
        **kwargs,
    )


@passes_kws_to(add_wkt)
def add_shapely_geometry(
    qgis_instance_handle: Any, geom: BaseGeometry, *args, **kwargs
) -> None:
    """
    Add a shapely geometry

    :param geom:
    :param qgis_instance_handle:
    :return:
    """

    add_wkt(qgis_instance_handle, geom.wkt, *args, **kwargs)


@passes_kws_to(add_shapely_geometry)
def add_shapely_geometries(
    qgis_instance_handle: Any, geometries: Mapping, *args, **kwargs
) -> None:
    """

    :param geometries:
    :param qgis_instance_handle:
    :param kwargs:
    :return:
    """
    for layer_name, geometry in geometries.items():
        add_shapely_geometry(
            qgis_instance_handle, geometry, name=layer_name, *args, **kwargs
        )


@passes_kws_to(add_wkt_layer)
def add_shapely_layer(
    qgis_instance_handle: Any,
    geoms: Iterable[shapely.geometry.base.BaseGeometry],
    *args,
    **kwargs,
) -> None:
    # assert geoms[0] == #TODO: SAME TYPE
    add_wkt_layer(qgis_instance_handle, [geom.wkt for geom in geoms], *args, **kwargs)


@passes_kws_to(add_wkb)  # OR add_wkt
def add_dataframe(
    qgis_instance_handle: Any,
    dataframe: DataFrame,
    geometry_column="geometry",
    *args,
    **kwargs,
) -> None:
    """

    :param qgis_instance_handle:
    :param dataframe:
    :param kwargs:
    :return:
    """
    from geopandas import GeoDataFrame
    from jord.geopandas_utilities import split_on_geom_type

    if isinstance(dataframe, GeoDataFrame):
        geom_dict = split_on_geom_type(dataframe)
        for df in geom_dict.values():
            if False:
                for w in df.geometry.to_wkt():
                    add_wkt(qgis_instance_handle, w, *args, **kwargs)
            else:
                for columns, w in zip(
                    df.iterrows(), df.geometry.to_wkb()
                ):  # TODO: ITERROWS may not work
                    add_wkb(qgis_instance_handle, w, columns, *args, **kwargs)

    elif isinstance(dataframe, DataFrame):
        raise NotImplemented
        if isinstance(
            dataframe[geometry_column][0], shapely.geometry.base.BaseGeometry
        ):
            a = dataframe[geometry_column][0]
            # if a.geom_type == "MultiPolygon":

            wkts = [d.wkt for d in dataframe[geometry_column]]
        elif isinstance(dataframe[geometry_column][0], str):
            wkts = dataframe[geometry_column]
        else:
            raise NotImplemented

        for row in wkts:
            add_wkt(qgis_instance_handle, row, *args, **kwargs)
    else:
        raise NotImplemented


@passes_kws_to(add_dataframe)
def add_dataframes(
    qgis_instance_handle: Any, dataframes: Mapping[str, DataFrame], *args, **kwargs
) -> None:
    for layer_name, geometry in dataframes.items():
        add_dataframe(qgis_instance_handle, geometry, name=layer_name, *args, **kwargs)


@passes_kws_to(add_wkt_layer)
def add_dataframe_layer(
    qgis_instance_handle: Any,
    dataframe: DataFrame,
    name: str = None,
    geometry_column="geometry",
    *args,
    **kwargs,
) -> None:
    from geopandas import GeoDataFrame
    from jord.geopandas_utilities import split_on_geom_type

    if isinstance(dataframe, GeoDataFrame):
        geom_dict = split_on_geom_type(dataframe)

        append_type_name = False

        if len(geom_dict) > 1:  # Must be split
            append_type_name = True
            if (
                name is None
            ):  # if No name give one to have some commoness of new layers in names
                name = DEFAULT_LAYER_NAME
                if APPEND_TIMESTAMP and False:
                    name += f"_{time.time()}"

        for k, df in geom_dict.items():  # each geom type
            geoms = []
            columns = []
            for (i, c), w in zip(df.iterrows(), df.geometry.to_wkt()):
                c.pop(geometry_column)
                geoms.append(w)
                columns.append({**c})

            if append_type_name:
                layer_name = f"{name}_{k.name}"
            else:
                layer_name = name

            add_wkt_layer(
                qgis_instance_handle,
                geoms,
                name=layer_name,
                columns=columns,
                *args,
                **kwargs,
            )
    elif isinstance(dataframe, DataFrame):
        raise NotImplemented
        geom_dict = split_on_geom_type(dataframe)
        for df in geom_dict.values():  # each geom type
            geoms = []
            columns = []
            for (i, c), w in zip(df.iterrows(), df.geometry.to_wkt()):
                c.pop(geometry_column)
                geoms.append(w)
                columns.append({**c})

            add_wkt_layer(qgis_instance_handle, geoms, columns=columns, *args, **kwargs)

    else:
        raise NotImplemented(f"{type(dataframe)}, {dataframe}")


@passes_kws_to(add_shapely_geometry)
def add_geojson(qgis_instance_handle: Any, geojson_: str, *args, **kwargs) -> None:
    """

    :param geojson_:
    :param qgis_instance_handle:
    :param kwargs:
    :return:
    """
    # meta_data = ''
    add_shapely_geometry(
        qgis_instance_handle, shapely.from_geojson(geojson_), *args, **kwargs
    )


@passes_kws_to(add_shapely_geometry)
def add_geojsons(
    qgis_instance_handle: Any, geojsons: Mapping[str, str], *args, **kwargs
) -> None:
    """

    :param qgis_instance_handle:
    :param geojsons:
    :param kwargs:
    :return:
    """
    for layer_name, geojson_ in geojsons.items():
        add_shapely_geometry(
            qgis_instance_handle,
            shapely.from_geojson(geojson_),
            name=layer_name,
            *args,
            **kwargs,
        )


@passes_kws_to(add_shapely_layer)
def add_geojson_layer(
    qgis_instance_handle: Any, geojsons: Iterable[str], *args, **kwargs
) -> None:
    add_shapely_layer(
        qgis_instance_handle,
        [shapely.from_geojson(geojson_) for geojson_ in geojsons],
        *args,
        **kwargs,
    )


def remove_layers(qgis_instance_handle: Any, *args) -> None:
    """
    clear all the added layers

    :param qgis_instance_handle:
    :param args:
    :return: None
    :rtype: None
    """
    qgis_instance_handle.on_clear_temporary()


def remove_layer(qgis_instance_handle: Any, name: str, *args) -> None:
    ...
    # qgis_instance_handle.on_clear_temporary()


def clear_all(qgis_instance_handle: Any, *args) -> None:  # TODO: REMOVE THIS!
    """
    clear all the added layers

    :param qgis_instance_handle:
    :return:
    """
    remove_layers(qgis_instance_handle)
    if VERBOSE:
        print("CLEAR ALL!")


def add_raster(
    qgis_instance_handle: Any,
    raster: numpy.ndarray,
    name: str = DEFAULT_LAYER_NAME,
    centroid: Tuple[Number, Number] = None,
    extent_tuple: Tuple[Number, Number, Number, Number] = None,
    pixel_size: Tuple[Number, Number] = PIXEL_SIZE,
    crs_str: str = DEFAULT_LAYER_CRS,
    default_value: Number = DEFAULT_NUMBER,
    field: str = None,
    no_data_value: int = -1,
) -> None:
    """
    add a raster

    :param no_data_value:
    :param qgis_instance_handle:
    :param raster:
    :param name:
    :param centroid:
    :param extent_tuple:
    :param pixel_size:
    :param crs_str:
    :param default_value:
    :param field:
    :return: None
    :rtype: None
    """
    # noinspection PyUnresolvedReferences
    from qgis.core import (
        QgsRectangle,
        QgsCoordinateReferenceSystem,
        QgsRasterBandStats,
        QgsSingleBandGrayRenderer,
        QgsMultiBandColorRenderer,
        QgsContrastEnhancement,
        QgsRasterLayer,
        QgsRasterFileWriter,
        Qgis,
    )
    from jord.qgis_utilities.numpy_utilities.data_type import get_qgis_type
    from jord.qgis_utilities import RasterDataProviderEditSession

    x_size, y_size, *rest_size = raster.shape

    if len(rest_size) == 0:
        raster = numpy.expand_dims(raster, axis=-1)

    *_, num_bands = raster.shape

    data_type = get_qgis_type(raster.dtype).value

    # QgsWkbTypes.displayString(gPolygon.wkbType())

    extent = QgsRectangle()

    if extent_tuple:
        extent.setXMinimum(extent_tuple[0])
        extent.setYMinimum(extent_tuple[1])
        extent.setXMaximum(extent_tuple[2])
        extent.setYMaximum(extent_tuple[3])
    else:
        raster_half_size = (PIXEL_SIZE * (x_size / 2.0), PIXEL_SIZE * (y_size / 2.0))

        raster_half_size = raster_half_size[1], raster_half_size[0]

        if centroid is None:
            centroid = (0, 0)  # raster_half_size

        extent.setXMinimum(centroid[0] - raster_half_size[0])
        extent.setXMaximum(centroid[0] + raster_half_size[0])

        extent.setYMinimum(centroid[1] - raster_half_size[1])
        extent.setYMaximum(centroid[1] + raster_half_size[1])

    if APPEND_TIMESTAMP:
        name += f"_{time.time()}"

    temp_file = (
        ensure_existence(PROJECT_APP_PATH.user_data / "rasters") / f"{uuid.uuid4()}.tif"
    )
    writer = QgsRasterFileWriter(str(temp_file))

    if num_bands > 1:
        provider = writer.createMultiBandRaster(
            dataType=data_type,
            width=x_size,
            height=y_size,
            extent=extent,
            crs=QgsCoordinateReferenceSystem(crs_str),
            nBands=num_bands,
        )
    else:
        provider = writer.createOneBandRaster(
            dataType=data_type,
            width=x_size,
            height=y_size,
            extent=extent,
            crs=QgsCoordinateReferenceSystem(crs_str),
        )

    w_pixels, h_pixels = (
        x_size,
        y_size,
    )  # TODO: FIGURE OUT HOW TO HANDLE NON SQUARE RASTERS! SCALE DIMS BY SOME AMOUNT.

    with RasterDataProviderEditSession(provider):
        progress = range(0, num_bands)

        if VERBOSE:
            progress = tqdm.tqdm(progress)

        for ith_band in progress:
            block = provider.block(
                bandNo=ith_band + 1, boundingBox=extent, width=w_pixels, height=h_pixels
            )
            provider.setNoDataValue(bandNo=ith_band + 1, noDataValue=no_data_value)

            for wp in range(0, w_pixels):
                for hp in range(0, h_pixels):
                    value = raster[wp][hp][ith_band]
                    if value == numpy.nan:
                        if block.setIsNoData(wp, hp):
                            continue
                    block.setValue(wp, hp, value)

            if VERBOSE:
                print("writing block on band", ith_band + 1)

            provider.writeBlock(block, band=ith_band + 1, xOffset=0, yOffset=0)

            del block

    layer = QgsRasterLayer(str(temp_file), name, "gdal")

    if num_bands == 1:
        # this is needed for the min and max value to refresh in the layer panel
        renderer = layer.renderer()

        gray_renderer = QgsSingleBandGrayRenderer(provider, 1)

        if CONTRAST_ENHANCE:
            stats = provider.bandStatistics(1, QgsRasterBandStats.All, extent)
            min_value = stats.minimumValue
            max_value = stats.maximumValue

            my_enhancement = QgsContrastEnhancement()
            my_enhancement.setContrastEnhancementAlgorithm(
                QgsContrastEnhancement.StretchToMinimumMaximum, True
            )
            my_enhancement.setMinimumValue(min_value)
            my_enhancement.setMaximumValue(max_value)
            gray_renderer.setContrastEnhancement(my_enhancement)

        layer.setRenderer(gray_renderer)

    elif num_bands != 4:
        multi_color_renderer = QgsMultiBandColorRenderer(provider, 1, 2, 3)

        layer.setRenderer(multi_color_renderer)
        layer.setDefaultContrastEnhancement()
        layer.triggerRepaint()
        # iface.legendInterface().refreshLayerSymbology(layer)

    else:
        multi_color_renderer = QgsMultiBandColorRenderer(provider, 1, 2, 3)

        layer.setRenderer(multi_color_renderer)
        layer.setDefaultContrastEnhancement()
        layer.triggerRepaint()

    if SKIP_MEMORY_LAYER_CHECK_AT_CLOSE:
        layer.setCustomProperty("skipMemoryLayersCheck", 1)

    qgis_instance_handle.qgis_project.addMapLayer(layer, False)
    qgis_instance_handle.temporary_group.insertLayer(0, layer)


@passes_kws_to(add_raster, no_pass_filter=["name"])
def add_rasters(qgis_instance_handle, rasters: Mapping, *args, **kwargs) -> None:
    """

    :param qgis_instance_handle:
    :param rasters:
    :param kwargs:
    :return:
    """
    for layer_name, raster in rasters.items():
        add_raster(qgis_instance_handle, raster, *args, name=layer_name, **kwargs)


class QliveRPCMethodEnum(Enum):
    # add_layers = add_layers.__name__

    remove_layers = remove_layers.__name__
    clear_all = clear_all.__name__
    remove_layer = remove_layer.__name__

    add_wkt = add_wkt.__name__
    add_wkts = add_wkts.__name__
    add_wkt_layer = add_wkt_layer.__name__

    add_wkb = add_wkb.__name__
    add_wkbs = add_wkbs.__name__
    add_wkb_layer = add_wkb_layer.__name__

    add_dataframe = add_dataframe.__name__
    add_dataframes = add_dataframes.__name__
    add_dataframe_layer = add_dataframe_layer.__name__

    add_shapely_geometry = add_shapely_geometry.__name__
    add_shapely_geometries = add_shapely_geometries.__name__
    add_shapely_layer = add_shapely_layer.__name__

    add_geojson = add_geojson.__name__
    add_geojsons = add_geojsons.__name__
    add_geojson_layer = add_geojson_layer.__name__

    add_raster = add_raster.__name__
    add_rasters = add_rasters.__name__


funcs = locals()  # In local scope for name
QliveRPCMethodMap = {e: funcs[e.value] for e in QliveRPCMethodEnum}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Lindbjerg"
__doc__ = r"""

           Created on 02-12-2020
           """

from typing import Any

import numpy
from qgis.PyQt import QtGui
from qgis.PyQt.QtCore import QEvent, Qt
from qgis.gui import QgsPixmapLabel

from .conversion import get_qimage_from_numpy

__all__ = ["NumpyImageWidget"]


class NumpyImageWidget(QgsPixmapLabel):
    """solely for the purpose of the maintaining the aspect ratio of the image"""

    def __init__(self, parent: Any = None):
        super().__init__(parent)
        # self.setScaledContents(True)
        # self.setSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        # self.setSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Minimum)
        # self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        # self.setSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
        # self.setSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        # self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        # self.setAlignment(Qt.AlignCenter)
        self.setText("Waiting for layer...")
        self.setToolTip("Waiting for layer...")
        # self.setCentralWidget(self.label)
        # self.label.adjustSize()
        # self.resize(self.pixmap.width(), self.pixmap.height())
        # self.installEventFilter(self)

    def recalculate_size(self) -> None:  # TODO: maybe reset to non scaled img
        """

        :return:
        """
        if False:
            self.pixmap = self.pixmap.scaled(
                self.size(), Qt.KeepAspectRatio, transformMode=Qt.SmoothTransformation
            )
        elif False:  #
            self.pixmap = self.pixmap.scaledToWidth(
                self.width(), mode=Qt.SmoothTransformation
            )
        elif False:  #
            self.pixmap = self.pixmap.scaledToHeight(
                self.height(), mode=Qt.SmoothTransformation
            )
        else:
            pass

        # self.setMinimumSize(img.shape[1], img.shape[0])
        # self.setMinimumSize(pixmap.width(), pixmap.height())
        # self.setMinimumSize(1,1)

        self.setPixmap(self.pixmap)

    def setImage(self, img: numpy.ndarray) -> None:
        """

        :param img:
        :return:
        """
        self.pixmap = QtGui.QPixmap.fromImage(get_qimage_from_numpy(img))
        self.recalculate_size()

    def eventFilter(self, source: Any, event: Any) -> Any:
        """

        :param source:
        :param event:
        :return:
        """
        if source is self and event.type() == QEvent.Resize:
            # print("resize")
            if False:
                if self.pixmap():
                    self.recalculate_size()
        return super().eventFilter(source, event)


if __name__ == "__main__":
    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


from shapely.geometry.base import BaseGeometry

__all__ = ["closing", "opening", "erode", "erosion", "dilate", "dilation", "close"]


def erosion(geom: BaseGeometry, eps: float = 1e-7) -> BaseGeometry:
    """

    :param geom: The geometry to be eroded
    :param eps: Erosion amount
    :return: The eroded geometry
    """
    return geom.buffer(-eps)


erode = erosion


def dilation(geom: BaseGeometry, eps: float = 1e-7) -> BaseGeometry:
    """

    :param geom: The geometry to be dilated
    :param eps: Dilation amount
    :return: The dilated geometry
    """
    return geom.buffer(eps)


dilate = dilation


def closing(geom: BaseGeometry, eps: float = 1e-7) -> BaseGeometry:
    """

    :param geom: The geometry to be closed
    :param eps: Dilation and Erosion amount
    :return: The closed geometry
    """
    return erode(dilate(geom, eps), eps)


close = closing


def opening(geom: BaseGeometry, eps: float = 1e-7) -> BaseGeometry:
    """

    :param geom: The geometry to be opened
    :param eps: Erosion and Dilation amount
    :return: The opened geometry
    """
    return dilate(erode(geom, eps), eps)


# open = opening # keyword clash

if __name__ == "__main__":

    def aishdjauisd():
        # Import constructors for creating geometry collections
        from shapely.geometry import MultiPoint, MultiLineString

        # Import necessary geometric objects from shapely module
        from shapely.geometry import Point, LineString, Polygon

        # Create Point geometric object(s) with coordinates
        point1 = Point(2.2, 4.2)
        point2 = Point(7.2, -25.1)
        point3 = Point(9.26, -2.456)
        # point3D = Point(9.26, -2.456, 0.57)

        # Create a MultiPoint object of our points 1,2 and 3
        multi_point = MultiPoint([point1, point2, point3])

        # It is also possible to pass coordinate tuples inside
        multi_point2 = MultiPoint([(2.2, 4.2), (7.2, -25.1), (9.26, -2.456)])

        # We can also create a MultiLineString with two lines
        line1 = LineString([point1, point2])
        line2 = LineString([point2, point3])
        multi_line = MultiLineString([line1, line2])
        polygon = Polygon([point2, point1, point3])

        from shapely.geometry import GeometryCollection
        from matplotlib import pyplot
        import geopandas

        geoms = GeometryCollection([multi_point, multi_point2, multi_line, polygon])

        # A positive distance produces a dilation, a negative distance an erosion. A very small or zero distance may sometimes be used to “tidy” a polygon.
        geoms = opening(geoms)
        geoms = dilate(geoms)
        geoms = closing(geoms)
        geoms = erode(geoms)

        p = geopandas.GeoSeries(geoms)
        p.plot()
        pyplot.show()

    aishdjauisd()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "heider"
__doc__ = r"""

           Created on 5/5/22
           """

from pathlib import Path

with open(Path(__file__).parent / "README.md", "r") as this_init_file:
    __doc__ += this_init_file.read()

try:
    from .conversion import *
    from .enums import *
    from .importing import *
    from .context import *
    from .cloning import *
    from .persistence import *
    from .spatial_reference import *
    from .drivers import *
    from .error_handling import *
except ImportError as ix:
    this_package_name = Path(__file__).parent.name
    print(f"Make sure gdal module is available for {this_package_name}")
    raise ix



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Lindbjerg"
__doc__ = r"""

           Created on 02-12-2020
           """

from typing import Tuple

# noinspection PyUnresolvedReferences
from qgis.PyQt import QtGui, QtWidgets

__all__ = ["dialog_progress_bar"]


def dialog_progress_bar(
    progress: int = 0, *, minimum_width: int = 300
) -> Tuple[QtWidgets.QDialog, QtWidgets.QProgressBar]:
    """
    Create a progress bar dialog.

    :param progress: The progress to display.
    :type progress: int
    :param minimum_width: The minimum width of the dialog.
    :type minimum_width: int
    :return: The dialog.
    :rtype: Tuple[QtWidgets.QDialog, QtWidgets.QProgressBar]
    """
    dialog = QtGui.QProgressDialog()
    dialog.setWindowTitle("Progress")
    dialog.setLabelText("text")

    bar = QtWidgets.QProgressBar(dialog)
    bar.setTextVisible(True)
    bar.setValue(progress)

    dialog.setBar(bar)
    dialog.setMinimumWidth(minimum_width)
    dialog.show()

    return dialog, bar


if __name__ == "__main__":

    def calc(x, y):
        from time import sleep

        dialog, bar = dialog_progress_bar(0)
        bar.setValue(0)
        bar.setMaximum(100)
        sum_ = 0
        for i in range(x):
            for j in range(y):
                k = i + j
                sum_ += k
            i += 1
            bar.setValue((float(i) / float(x)) * 100)
            sleep(0.1)
        print(sum_)

    # calc(10000, 2000)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Lindbjerg"
__doc__ = r"""

           Created on 02-12-2020
           """

__all__ = ["store_plugin_setting", "read_plugin_setting"]

from typing import Any

from jord import PROJECT_NAME


def store_plugin_setting(
    key: str, value: Any, *, project_name: str = PROJECT_NAME
) -> None:
    """

    :param key:
    :param value:
    :param project_name:
    :return:
    """
    # noinspection PyUnresolvedReferences
    from qgis.core import QgsSettings

    QgsSettings().setValue(f"{project_name}/{key}", value)


def read_plugin_setting(
    key: str, *, default_value: Any = None, project_name: str = PROJECT_NAME
) -> Any:
    """

    :param key:
    :param default_value:
    :param project_name:
    :return:
    """
    # noinspection PyUnresolvedReferences
    from qgis.core import QgsSettings

    return QgsSettings().value(f"{project_name}/{key}", default_value)


if __name__ == "__main__":
    store_plugin_setting("mytext", "hello world")
    print(read_plugin_setting("mytext"))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


from enum import Enum

__all__ = ["CommonVectorDataProviderLibEnum"]


class CommonVectorDataProviderLibEnum(Enum):
    memory = "memory"
    postgres = "postgres"


def auishd(iface) -> None:
    """
    Some services require you to supply your own API key for the services to work.

    For Esri basemaps you will need a valid ArcGIS online subscription to use the maps.
    """
    from PyQt5.QtCore import QSettings

    """
  connection_type
  connection_name
  authcfg
  password
  referer
  url
  username
  zmax
  zmin
"""

    # Sources
    sources = []
    sources.append(
        [
            "connections-xyz",
            "Google Maps",
            "",
            "",
            "",
            "https://mt1.google.com/vt/lyrs=m&x=%7Bx%7D&y=%7By%7D&z=%7Bz%7D",
            "",
            "19",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Google Satellite",
            "",
            "",
            "",
            "https://mt1.google.com/vt/lyrs=s&x=%7Bx%7D&y=%7By%7D&z=%7Bz%7D",
            "",
            "19",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Google Terrain",
            "",
            "",
            "",
            "https://mt1.google.com/vt/lyrs=t&x=%7Bx%7D&y=%7By%7D&z=%7Bz%7D",
            "",
            "19",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Google Terrain Hybrid",
            "",
            "",
            "",
            "https://mt1.google.com/vt/lyrs=p&x=%7Bx%7D&y=%7By%7D&z=%7Bz%7D",
            "",
            "19",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Google Satellite Hybrid",
            "",
            "",
            "",
            "https://mt1.google.com/vt/lyrs=y&x=%7Bx%7D&y=%7By%7D&z=%7Bz%7D",
            "",
            "19",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Stamen Terrain",
            "",
            "",
            "Map tiles by Stamen Design, under CC BY 3.0. Data by OpenStreetMap, under ODbL",
            "http://tile.stamen.com/terrain/%7Bz%7D/%7Bx%7D/%7By%7D.png",
            "",
            "20",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Stamen Toner",
            "",
            "",
            "Map tiles by Stamen Design, under CC BY 3.0. Data by OpenStreetMap, under ODbL",
            "http://tile.stamen.com/toner/%7Bz%7D/%7Bx%7D/%7By%7D.png",
            "",
            "20",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Stamen Toner Light",
            "",
            "",
            "Map tiles by Stamen Design, under CC BY 3.0. Data by OpenStreetMap, under ODbL",
            "http://tile.stamen.com/toner-lite/%7Bz%7D/%7Bx%7D/%7By%7D.png",
            "",
            "20",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Stamen Watercolor",
            "",
            "",
            "Map tiles by Stamen Design, under CC BY 3.0. Data by OpenStreetMap, under ODbL",
            "http://tile.stamen.com/watercolor/%7Bz%7D/%7Bx%7D/%7By%7D.jpg",
            "",
            "18",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Wikimedia Map",
            "",
            "",
            "OpenStreetMap contributors, under ODbL",
            "https://maps.wikimedia.org/osm-intl/%7Bz%7D/%7Bx%7D/%7By%7D.png",
            "",
            "20",
            "1",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Wikimedia Hike Bike Map",
            "",
            "",
            "OpenStreetMap contributors, under ODbL",
            "http://tiles.wmflabs.org/hikebike/%7Bz%7D/%7Bx%7D/%7By%7D.png",
            "",
            "17",
            "1",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Esri Boundaries Places",
            "",
            "",
            "Requires ArcGIS Onlinesubscription",
            "https://server.arcgisonline.com/ArcGIS/rest/services/Reference/World_Boundaries_and_Places/MapServer/tile/%7Bz%7D/%7By%7D/%7Bx%7D",
            "",
            "20",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Esri Gray (dark)",
            "",
            "",
            "Requires ArcGIS Onlinesubscription",
            "http://services.arcgisonline.com/ArcGIS/rest/services/Canvas/World_Dark_Gray_Base/MapServer/tile/%7Bz%7D/%7By%7D/%7Bx%7D",
            "",
            "16",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Esri Gray (light)",
            "",
            "",
            "Requires ArcGIS Onlinesubscription",
            "http://services.arcgisonline.com/ArcGIS/rest/services/Canvas/World_Light_Gray_Base/MapServer/tile/%7Bz%7D/%7By%7D/%7Bx%7D",
            "",
            "16",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Esri National Geographic",
            "",
            "",
            "Requires ArcGIS Onlinesubscription",
            "http://services.arcgisonline.com/ArcGIS/rest/services/NatGeo_World_Map/MapServer/tile/%7Bz%7D/%7By%7D/%7Bx%7D",
            "",
            "12",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Esri Ocean",
            "",
            "",
            "Requires ArcGIS Onlinesubscription",
            "https://services.arcgisonline.com/ArcGIS/rest/services/Ocean/World_Ocean_Base/MapServer/tile/%7Bz%7D/%7By%7D/%7Bx%7D",
            "",
            "10",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Esri Satellite",
            "",
            "",
            "Requires ArcGIS Onlinesubscription",
            "https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/%7Bz%7D/%7By%7D/%7Bx%7D",
            "",
            "17",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Esri Standard",
            "",
            "",
            "Requires ArcGIS Onlinesubscription",
            "https://server.arcgisonline.com/ArcGIS/rest/services/World_Street_Map/MapServer/tile/%7Bz%7D/%7By%7D/%7Bx%7D",
            "",
            "17",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Esri Terrain",
            "",
            "",
            "Requires ArcGIS Onlinesubscription",
            "https://server.arcgisonline.com/ArcGIS/rest/services/World_Terrain_Base/MapServer/tile/%7Bz%7D/%7By%7D/%7Bx%7D",
            "",
            "13",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Esri Transportation",
            "",
            "",
            "Requires ArcGIS Onlinesubscription",
            "https://server.arcgisonline.com/ArcGIS/rest/services/Reference/World_Transportation/MapServer/tile/%7Bz%7D/%7By%7D/%7Bx%7D",
            "",
            "20",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Esri Topo World",
            "",
            "",
            "Requires ArcGIS Onlinesubscription",
            "http://services.arcgisonline.com/ArcGIS/rest/services/World_Topo_Map/MapServer/tile/%7Bz%7D/%7By%7D/%7Bx%7D",
            "",
            "20",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "OpenStreetMap Standard",
            "",
            "",
            "OpenStreetMap contributors, under ODbL",
            "http://tile.openstreetmap.org/%7Bz%7D/%7Bx%7D/%7By%7D.png",
            "",
            "19",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "OpenStreetMap H.O.T.",
            "",
            "",
            "OpenStreetMap contributors, under ODbL",
            "http://tile.openstreetmap.fr/hot/%7Bz%7D/%7Bx%7D/%7By%7D.png",
            "",
            "19",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "OpenStreetMap Monochrome",
            "",
            "",
            "OpenStreetMap contributors, under ODbL",
            "http://tiles.wmflabs.org/bw-mapnik/%7Bz%7D/%7Bx%7D/%7By%7D.png",
            "",
            "19",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "OpenTopoMap",
            "",
            "",
            "Kartendaten: © OpenStreetMap-Mitwirkende, SRTM | Kartendarstellung: © OpenTopoMap (CC-BY-SA)",
            "https://tile.opentopomap.org/%7Bz%7D/%7Bx%7D/%7By%7D.png",
            "",
            "17",
            "1",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Strava All",
            "",
            "",
            "OpenStreetMap contributors, under ODbL",
            "https://heatmap-external-b.strava.com/tiles/all/bluered/%7Bz%7D/%7Bx%7D/%7By%7D.png",
            "",
            "15",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Strava Run",
            "",
            "",
            "OpenStreetMap contributors, under ODbL",
            "https://heatmap-external-b.strava.com/tiles/run/bluered/%7Bz%7D/%7Bx%7D/%7By%7D.png?v=19",
            "",
            "15",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Open Weather Map Temperature",
            "",
            "",
            "Map tiles by OpenWeatherMap, under CC BY-SA 4.0",
            "http://tile.openweathermap.org/map/temp_new/%7Bz%7D/%7Bx%7D/%7By%7D.png?APPID={your_API_key}",
            "",
            "19",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Open Weather Map Clouds",
            "",
            "",
            "Map tiles by OpenWeatherMap, under CC BY-SA 4.0",
            "http://tile.openweathermap.org/map/clouds_new/%7Bz%7D/%7Bx%7D/%7By%7D.png?APPID={your_API_key}",
            "",
            "19",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Open Weather Map Wind Speed",
            "",
            "",
            "Map tiles by OpenWeatherMap, under CC BY-SA 4.0",
            "http://tile.openweathermap.org/map/wind_new/%7Bz%7D/%7Bx%7D/%7By%7D.png?APPID={your_API_key}",
            "",
            "19",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "CartoDb Dark Matter",
            "",
            "",
            "Map tiles by CartoDB, under CC BY 3.0. Data by OpenStreetMap, under ODbL.",
            "http://basemaps.cartocdn.com/dark_all/%7Bz%7D/%7Bx%7D/%7By%7D.png",
            "",
            "20",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "CartoDb Positron",
            "",
            "",
            "Map tiles by CartoDB, under CC BY 3.0. Data by OpenStreetMap, under ODbL.",
            "http://basemaps.cartocdn.com/light_all/%7Bz%7D/%7Bx%7D/%7By%7D.png",
            "",
            "20",
            "0",
        ]
    )
    sources.append(
        [
            "connections-xyz",
            "Bing VirtualEarth",
            "",
            "",
            "",
            "http://ecn.t3.tiles.virtualearth.net/tiles/a{q}.jpeg?g=1",
            "",
            "19",
            "1",
        ]
    )

    for source in sources:  # Add sources to browser
        connection_type = source[0]
        connection_name = source[1]

        QSettings().setValue(
            f"qgis/{connection_type}/{connection_name}/authcfg", source[2]
        )
        QSettings().setValue(
            f"qgis/{connection_type}/{connection_name}/password", source[3]
        )
        QSettings().setValue(
            f"qgis/{connection_type}/{connection_name}/referer", source[4]
        )
        QSettings().setValue(f"qgis/{connection_type}/{connection_name}/url", source[5])
        QSettings().setValue(
            f"qgis/{connection_type}/{connection_name}/username", source[6]
        )
        QSettings().setValue(
            f"qgis/{connection_type}/{connection_name}/zmax", source[7]
        )
        QSettings().setValue(
            f"qgis/{connection_type}/{connection_name}/zmin", source[8]
        )

    iface.reloadConnections()  # Update GUI



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__all__ = ["reflection"]

from numbers import Number
from typing import Callable

from shapely.affinity import scale
from shapely.geometry import Polygon
from shapely.ops import transform


# pts = numpy.array([[0, 0], [1, 1], [1, 2], [0, 1]])
# display(Polygon(pts.dot([[1,0],[0,-1]])))


def principal_axis():
    ...


def reflection(x0: Number) -> Callable:
    """

    :param x0:
    :return:
    """
    return lambda x, y: (2 * x0 - x, y)


if __name__ == "__main__":

    def ausdhuashd():
        P = Polygon([[0, 0], [1, 1], [1, 2], [0, 1]])
        print(P)  # POLYGON ((0 0, 1 1, 1 2, 0 1, 0 0))

        Q1 = scale(P, xfact=-1, origin=(1, 0))  # X-axis
        # scale(P, yfact = -1, origin = (1, 0)) # Y-axis
        Q2 = transform(reflection(1), P)

        print(Q1)  # POLYGON ((2 0, 1 1, 1 2, 2 1, 2 0))
        print(Q2)  # POLYGON ((2 0, 1 1, 1 2, 2 1, 2 0))

    ausdhuashd()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "heider"
__doc__ = r"""

           Created on 1/23/23
           """

from pathlib import Path

with open(Path(__file__).parent / "README.md", "r") as this_init_file:
    __doc__ += this_init_file.read()

from .clamp import *
from .lines import *
from .morphology import *
from .points import *
from .polygons import *
from .serialisation import *
from .geometry_types import *
from .projection import *
from .transformation import *
from .rings import *
from .selection import *



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Lindbjerg"

from warg import ensure_in_sys_path, find_nearest_ancestral_relative

ensure_in_sys_path(find_nearest_ancestral_relative("jord").parent)


def test_import_package():
    if True:
        import jord

        print(jord.__version__)


def test_qgis_import_package():
    if False:
        from jord import qgis_utilities

        print(qgis_utilities.__doc__)


def test_gdal_import_package():
    if False:
        from jord import gdal_utilities

        print(gdal_utilities.__doc__)


def test_qlive_import_package():
    if True:
        from jord import qlive_utilities

        print(qlive_utilities.__doc__)


def test_geopandas_import_package():
    if True:
        from jord import geopandas_utilities

        print(geopandas_utilities.__doc__)


def test_geojson_import_package():
    if True:
        from jord import geojson_utilities

        print(geojson_utilities.__doc__)


def test_pillow_import_package():
    if True:
        from jord import pillow_utilities

        print(pillow_utilities.__doc__)


def test_qt_import_package():
    if False:
        from jord import qt_utilities

        print(qt_utilities.__doc__)


def test_pil_import_package():
    if True:
        from jord import pillow_utilities

        print(pillow_utilities.__doc__)


def test_shapely_import_package():
    if True:
        from jord import shapely_utilities

        print(shapely_utilities.__doc__)


def test_rasterio_import_package():
    if True:
        from jord import rasterio_utilities

        print(rasterio_utilities.__doc__)


def test_torch_import_package():
    if False:
        from jord.exclude import torch_utilities

        print(torch_utilities.__doc__)


if __name__ == "__main__":
    test_gdal_import_package()
    test_pil_import_package()
    test_gdal_import_package()
    test_import_package()
    test_shapely_import_package()
    test_rasterio_import_package()
